/* NessusJ
 * Copyright (C) 1999 Alexis de Bernis <alexisb@cvs.nessus.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This program is a nessusd java client. For any information about
 * the nessus project visit <http://www.nessus.org>
 *
 *      PLEASE READ CAREFULLY NESSUS DOCUMENTATION BEFORE
 *                     USING THIS PROGRAM
 *
 */

/* importation directives has changed in newer JDK, uncomment
 * the ones you use if you have compilation troubles :
 * JDK up to 1.1.7 (Swing 1.0.x) : classes in com.sun.java.swing
 * JDK from 1.2 (Swing 1.1.x) : classes in javax.swing
 * You must change this in every .java file...
 */

import java.lang.*;
import java.io.*;

import com.sun.java.swing.*;
//import javax.swing.*;

class Listener extends Thread
{
    private DataInputStream from_nessusd;
    private DataOutputStream for_nessusd;
    private boolean finished;
    private Status status;

    Listener(Status status,DataInputStream from_nessusd,
                                         DataOutputStream for_nessusd)
    {
        super("NessusdListener");

        this.from_nessusd=from_nessusd;
        this.for_nessusd=for_nessusd;
        this.status=status;
        finished=false;
    }

    public void run()
    {
//*        System.out.println("NessusdListener launched");

        while(!finished)
        {
            wait_for_message();
        }
    }

    private void wait_for_message()
    {
        String received;

        try
        {
			received=from_nessusd.readLine();
        }catch(IOException e)
        {
            JOptionPane.showMessageDialog(status,"Error waiting for message","Error",JOptionPane.ERROR_MESSAGE);
            finished=true;
            return;
        }

        dispatch_message(received);
    }

    private void dispatch_message(String message)
    {

		String parts[]=parse_ntp_msg(message);

        if(!parts[0].equals("SERVER"))
        {
            System.out.println("Listener :: dispatch_message : Message is not from server, header : "+parts[0]);
            return;
        }

        if(parts[1].equals("HOLE"))
        {
			String description=parts[4].replace(';','\n');
			status.post_hole(parts[2],new Hole(parts[3],description));
            return;
        }

        if(parts[1].equals("INFO"))
        {
			String description=parts[4].replace(';','\n');
			status.post_info(parts[2],new Info(parts[3],description));
            return;
        }

        if(parts[1].equals("STATUS"))
        {
            status.refresh_host(parts[2],parts[3],parts[4]);
            return;
        }

        if(parts[1].equals("PORT"))
        {
			status.post_port(parts[2],parts[3]);
            return;
        }

        if(parts[1].equals("BYE"))
        {
            finished=true;
            status.jobFinished();
            return;
        }

        if(parts[1].equals("ERROR"))
        {
            System.out.println("Listener :: dispatch_message : error message received : "+parts[2]);
            return;
        }

        if(parts[1].equals("PING"))
        {
            sendln("CLIENT <|> PONG <|> PING <|> CLIENT");
            return;
        }

        System.out.println("Listener :: dispatch_message : unknown message type "+parts[1]);
        return;        
    }

	public boolean send(String msg)
	{
		try
		{
			for_nessusd.writeBytes(msg);
		} catch(Exception e)
		{
			System.out.println("Listener :: send : error sending"+msg);
			return(false);
		}
		return(true);
	}

	public boolean sendln(String msg)
	{
		try
		{
			for_nessusd.writeBytes(msg+"\n");
		} catch(Exception e)
		{
			System.out.println("Listener :: send : error sending"+msg);
			return(false);
		}
		return(true);
	}

	public String read_line()
	{
		String msg;

		try
		{
			msg=from_nessusd.readLine();
		} catch(Exception e)
		{
			System.out.println("Listener :: read_line : "+e);
			return(null);
		}
		return(msg);
	}


    private String[] parse_ntp_msg(String msg)
    {
        int pos=-5,num=0;

//		System.out.println("NessusJ :: parse_ntp_msg : parsing "+msg);

        while((pos=msg.indexOf(" <|> ",pos+5))!=-1)
		    num++;
			
        String[] res=new String[num+1];

        res[0]=msg.substring(0,(pos=msg.indexOf(" <|> ")));
        res[num]=msg.substring(msg.lastIndexOf(" <|> ")+5);

        for(int i=1;i<num;i++)
		{
//			System.out.println("NessusJ :: parse_ntp_msg : pos="+pos+" num="+num);
            res[i]=msg.substring(pos+5,(pos=msg.indexOf(" <|> ",pos+5)));
		}
		
        return(res);
    }

	private String make_ntp_msg(String[] parts)
	{
		String msg="";

		for(int i=0;i<(parts.length-1);i++)
			msg=msg+parts[i]+" <|> ";

		msg=msg+parts[parts.length-1];

		return(msg);
	}


}


