/* NessusJ 
 * Copyright (C) 1999 Alexis de Bernis <alexisb@cvs.nessus.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This program is a nessusd java client. For any information about
 * the nessus project visit <http://www.nessus.org>
 *
 *      PLEASE READ CAREFULLY NESSUS DOCUMENTATION BEFORE
 *                     USING THIS PROGRAM
 *
 */


/* importation directives has changed in newer JDK, uncomment
 * the ones you use if you have compilation troubles :
 * JDK up to 1.1.7 (Swing 1.0.x) : classes in com.sun.java.swing
 * JDK from 1.2 (Swing 1.1.x) : classes in javax.swing
 * You must change this in every .java file...
 */

import java.lang.*;
import java.net.*;
import java.io.*;
import java.util.*;

import com.sun.java.swing.*;
//import javax.swing.*;

public class NessusJ
{
	private PicsLib pics;
	private NessusJUI gui;
	private Preferences prefs;
	private Hashtable families,server_prefs;
	private Vector plugins,rules;
	private String[] plugins_order;

	private Status test_status;
	private Report test_report;
	
	private DataInputStream from_nessusd;
	private DataOutputStream for_nessusd;

	
	NessusJ()
	{
		pics=new PicsLib(false); /* load pics while they are asked */
		
		Intro nessus_intro=new Intro(this);
		nessus_intro.setState("Loading preferences...");

        prefs=new Preferences(".nessusrc");

		nessus_intro.setState("Loading Swing...");
		gui=new NessusJUI(pics,this);
        nessus_intro.dispose();
        nessus_intro=null;
	
		plugins=new Vector();
		families=new Hashtable();
	
	}
	
	public PicsLib get_pics_lib()
	{
		return(pics);
	}

	public NessusJUI get_gui()
	{
		return(gui);
	}

/** Parses the command-line args ( as -v) and instanciate NessusJ */

    public static void main(String args[])
    {
        for(int i=0;;i++)
        {
            try
            {
                if(args[i].equalsIgnoreCase("-v"))
                {
                    System.out.println("\nNessusJ for nessus alpha2-fix4 -- (c) A. de Bernis <alexisb@cvs.nessus.org>\n");
                    System.exit(0);
                }
            }
            catch(IndexOutOfBoundsException e)
            {
                break;
            }
        }

        new NessusJ();
    }

	public void quit_nessusj()
	{
//		System.out.println("NessusJ :: quit_nessusj : quitting");
		
		prefs.finalize(); /* Write prefs to a file */
		System.exit(0);
	}

    private boolean make_plugin_table()
    {
        String received=new String("");
        int id;
        String name,copyright,description,summary,type,family;

//        System.out.println("NessusJ :: make_plugin_table : Making plug-in table...");

        for(;;)
        {
			if((received=read_line())==null)
				return false;

//            System.out.println("NessusJ :: make_plugin_table : "+received);
            if(received.equals("<|> SERVER"))
                break;

			String parts[]=parse_ntp_msg(received);

            id=Integer.parseInt(parts[0]);
//            System.out.println("NessusJ :: make_plugin_table : New Plug-in: "+id);

            name=parts[1];
            type=parts[2];
            copyright=parts[3];
            description=parts[4];
            summary=parts[5];
            description=description.replace(';','\n');
            family=parts[6];

//            System.out.println("NessusJ :: make_plugin_table : Name :        "+name);
//            System.out.println("NessusJ :: make_plugin_table : Copyright :   "+copyright);
//            System.out.println("NessusJ :: make_plugin_table : Description : "+description);
//            System.out.println("NessusJ :: make_plugin_table : Summary :     "+summary);
//            System.out.println("NessusJ :: make_plugin_table : Type    :     "+type);
//            System.out.println("NessusJ :: make_plugin_table : Family  :     "+family+"\n");

            PlugIn temp=new PlugIn(id,
                                   name,
                                   copyright,
                                   description,
                                   summary,
                                   type,
                                   family,
                                   prefs.isPluginActivated(name),
                                   prefs,
								   this);

            plugins.addElement(temp);
            register_family(temp,family);

        }
            
//        System.out.println("NessusJ :: make_plugin_table : Finished to make plug-in table");
		gui.update_family_pane(families);
        return(get_server_prefs());
    }

    public boolean connect_nessusd(String host,int port,String login,String password)
    {
        String received;

//        System.out.println("NessusJ :: connect_nessusd : Connecting Nessusd:");
//        System.out.println("NessusJ :: connect_nessusd : Host:   "+host);
//        System.out.println("NessusJ :: connect_nessusd : Port:   "+port);
//        System.out.println("NessusJ :: connect_nessusd : login:  "+login);
//        System.out.println("NessusJ :: connect_nessusd : passwd: "+password+"\n");

        try
        {
            Socket connection=new Socket(host,port);
            from_nessusd=new DataInputStream(new BufferedInputStream(connection.getInputStream()));
            for_nessusd=new DataOutputStream(connection.getOutputStream());
        }
        catch(Exception e)
        {
            gui.disp_error_dialog("Error: Can't connect, bad host or port closed");
            return(false);
        }

//        System.out.println("NessusJ :: connect_nessusd : Connection OK!");
	
        sendln("< NTP/1.1 >");
        received=read_line();
//	    System.out.println("NessusJ :: connect_nessusd : Received : "+received);

        if(!received.equals("< NTP/1.1 >"))
        {
            gui.disp_error_dialog("Obsolete or bad server"+received);
            return(false);
        }
        
        sendln(login);
//        System.out.println("NessusJ :: connect_nessusd : Send : "+login);
        sendln(password);
//        System.out.println("NessusJ :: connect_nessusd : Send : "+password);
        received=read_line();
        
		if(!received.equals("User : Password : SERVER <|> PLUGIN_LIST <|>"))
        {
            gui.disp_error_dialog("Error: unexpected response: "+received+"\nMay be a bad login or password");
            return(false);
        }
         return(make_plugin_table());
	}

	private boolean get_server_rules()
	{
		String received;
		
		rules=prefs.get_rules();
		
		if(!(received=read_line()).equals("SERVER <|> RULES <|>"))
		{
			System.out.println("NessusJ :: get_server_rules : unexpected reply "+received);
			return(false);
		}
		
		while(!(received=read_line()).equals("<|> SERVER"))
		{
		 	boolean test=received.substring(0,1).equals("y");
			String host=received.substring(2,received.length()-1);
			Rule tmp=new Rule(received,true);
			rules.addElement(tmp);
//			System.out.println("NessusJ :: get_server_rules : new server rule "+tmp.toString());
		}

		gui.update_rules_list(rules);

		return(true);
	}

	public void register_family(PlugIn plugin, String family)
    {
        Family temp;

        temp=(Family) families.get(family);

        if(temp==null)
        {
            temp=new Family(family,this);
            families.put(family,temp);
        }

        temp.add_child(plugin);
    }

    private boolean get_server_prefs()
    {
        server_prefs=new Hashtable();
        String received=read_line();

//		System.out.println("NessusJ :: get_server_prefs : making server prefs");

        if(!received.equals("SERVER <|> PREFERENCES <|>"))
            return(false);

        while(!(received=read_line()).equals("<|> SERVER"))
        {
            int pos;
			if(received.indexOf("[")!=-1 && received.indexOf(":")!=-1)  // must be a plugin pref!
			{
				String plugin_name=received.substring(0,received.indexOf("["));
				get_plugin_by_name(plugin_name).add_plugin_pref(received.substring(received.indexOf("[")));
				continue;
			}


            server_prefs.put(received.substring(0,(pos=received.indexOf(" <|> "))),received.substring(pos+5));
        }

//		System.out.println("NessusJ :: get_server_prefs : finished");

		gui.update_server_prefs(server_prefs);

        return(get_server_rules());
    }

    private String[] parse_ntp_msg(String msg)
    {
        int pos=-5,num=0;

//		System.out.println("NessusJ :: parse_ntp_msg : parsing "+msg);

        while((pos=msg.indexOf(" <|> ",pos+5))!=-1)
		    num++;
			
        String[] res=new String[num+1];

        res[0]=msg.substring(0,(pos=msg.indexOf(" <|> ")));
        res[num]=msg.substring(msg.lastIndexOf(" <|> ")+5);

        for(int i=1;i<num;i++)
		{
//			System.out.println("NessusJ :: parse_ntp_msg : pos="+pos+" num="+num);
            res[i]=msg.substring(pos+5,(pos=msg.indexOf(" <|> ",pos+5)));
		}
		
        return(res);
    }

	private String make_ntp_msg(String[] parts)
	{
		String msg="";

		for(int i=0;i<(parts.length-1);i++)
			msg=msg+parts[i]+" <|> ";

		msg=msg+parts[parts.length-1];

		return(msg);
	}

	public boolean send(String msg)
	{
		try
		{
			for_nessusd.writeBytes(msg);
		} catch(Exception e)
		{
			System.out.println("NessusJ :: send : error sending"+msg);
			return(false);
		}
		return(true);
	}

	public boolean sendln(String msg)
	{
		try
		{
			for_nessusd.writeBytes(msg+"\n");
		} catch(Exception e)
		{
			System.out.println("NessusJ :: send : error sending"+msg);
			return(false);
		}
		return(true);
	}

	public String read_line()
	{
		String msg;

		try
		{
			msg=from_nessusd.readLine();
		} catch(Exception e)
		{
			System.out.println("NessusJ :: recv_line : "+e);
			return(null);
		}
		return(msg);
	}

	public boolean start_attack(String host)
	{
		return(send_local_prefs());
	}

	private boolean send_local_prefs()
	{
		Hashtable prefs_to_send=gui.get_prefs_to_send(server_prefs);
		Enumeration keys=prefs_to_send.keys();
		Enumeration values=prefs_to_send.elements();
		String parts[]=new String[2];
		parts[0]="CLIENT";
		parts[1]="PREFERENCES";

		sendln(make_ntp_msg(parts));

		while(keys.hasMoreElements())
		{
			parts[0]=(String) keys.nextElement();
			parts[1]=(String) values.nextElement();
			sendln(make_ntp_msg(parts));
		}

		/* Now we are to send Plugins Prefs */
		for(int i=0;i<plugins.size();i++)
		{
			PlugIn current=(PlugIn) plugins.elementAt(i);
			PluginPrefs current_prefs;

			if((current_prefs=current.get_plugin_prefs())==null)
				continue;	// No plugins preferences for this one

			send(current_prefs.get_user_prefs()); // send them !
		}

		parts[0]="";
		parts[1]="CLIENT";
		sendln(make_ntp_msg(parts));
		
		return(check_prefs_errors());
	}

	private boolean check_prefs_errors()
	{
		String received=read_line();
		
		if(!received.equals("SERVER <|> PREFERENCES_ERRORS <|>"))
		{
			System.out.println("NessusJ :: check_prefs_errors : header unknown : "+received);
			return(false);
		}
	
		while(!(received=read_line()).equals("<|> SERVER"))
		{
			String[] temp=parse_ntp_msg(received);
			gui.disp_error_dialog("Preferences error, for "+temp[0]+" server will use "+temp[1]);
		}
		
		return(send_rules());
	}

	private boolean send_rules()
	{
		Vector rules=gui.get_rules();		
		String parts[]=new String[2];
		parts[0]="CLIENT";
		parts[1]="RULES";

		sendln(make_ntp_msg(parts));

		for(int i=0;i<rules.size();i++)
		{
			sendln(((Rule) rules.elementAt(i)).toString());
		}

		parts[0]="";
		parts[1]="CLIENT";
		sendln(make_ntp_msg(parts));

		return(send_new_attack());
	}

	private boolean send_new_attack()
	{
		String[] parts=new String[4];
		parts[0]="CLIENT";
		parts[1]="NEW_ATTACK";
		parts[2]=gui.get_host_to_test();
		parts[3]="CLIENT";
		
		sendln(make_ntp_msg(parts));
		
		return(get_plugins_order());
	}

	private boolean get_plugins_order()
	{

		String received[]=parse_ntp_msg(read_line());
		
		if(!received[1].equals("PLUGINS_ORDER"))
		{
			System.out.println("NessusJ :: get_plugins_order : unexpected header : "+received[1]);
			return(false);
		}
		
		String order=received[2];
		plugins_order=new String[plugins.size()];
		int count=0;
		
		while(!order.equals(""))
		{
			int i=Integer.parseInt(order.substring(0,order.indexOf(";")));
			plugins_order[count]=((PlugIn) plugins.elementAt(i)).getName();
			count++;
			order=order.substring(order.indexOf(";")+1);
		}
		
		return(start_status_display());
	}

	private boolean start_status_display()
	{
		test_status=new Status(this,from_nessusd,for_nessusd,gui.get_host_to_test(),plugins_order);
		return(true);
	}

	public Vector get_plugins()
	{
		return(plugins);
	}

	private PlugIn get_plugin_by_name(String name)
	{
		for(int i=0;i<plugins.size();i++)
		{
			if(((PlugIn) plugins.elementAt(i)).getName().equals(name))
				return ((PlugIn) plugins.elementAt(i));
		}

		return(null);
	}
}
