/* NessusJ
 * Copyright (C) 1999 Alexis de Bernis <alexisb@cvs.nessus.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This program is a nessusd java client. For any information about
 * the nessus project visit <http://www.nessus.org>
 *
 *      PLEASE READ CAREFULLY NESSUS DOCUMENTATION BEFORE
 *                     USING THIS PROGRAM
 *
 */

/* importation directives has changed in newer JDK, uncomment
 * the ones you use if you have compilation troubles :
 * JDK up to 1.1.7 (Swing 1.0.x) : classes in com.sun.java.swing
 * JDK from 1.2 (Swing 1.1.x) : classes in javax.swing
 * You must change this in every .java file...
 */

import java.lang.*;
import java.awt.*;
import java.awt.event.*;

import com.sun.java.swing.*;
import com.sun.java.swing.border.*;
//import javax.swing.*;
//import javax.swing.border.*;


class PlugIn extends JPanel implements ActionListener
{
    private int id;
    private String name;
    private String copyright;
    private String description;
    private String summary;
    private String type;
    private String family;
    private JCheckBox state;
    private Preferences prefs;
    private JButton nameToClick,have_prefs;
	private NessusJ parent;
	private NessusJUI gui;
	private PluginPrefs plugin_prefs;
	private PicsLib pics;

    PlugIn(int id,String name,String copyright,String description,
           String summary,String type,String family,boolean actived,
           Preferences prefs,NessusJ parent)
    {
        super();

        this.id=id;
        this.name=name;
        this.copyright=copyright;
        this.description=description;
        this.summary=summary;
        this.type=type;
        this.family=family;
        this.prefs=prefs;
		this.parent=parent;
		gui=parent.get_gui();
		pics=parent.get_pics_lib();
		plugin_prefs=null;

        setLayout(new BorderLayout());
        state=new JCheckBox("",actived);
        state.addActionListener(this);
        state.setActionCommand("state");
        state.setIcon(pics.get_pic("flag_red"));
        state.setSelectedIcon(pics.get_pic("flag_green"));
        nameToClick=new JButton(name);
        nameToClick.setBorder(new EmptyBorder(0,0,0,0));
        nameToClick.setToolTipText(summary);
        nameToClick.addActionListener(this);
		nameToClick.setHorizontalAlignment(JButton.LEFT);
		have_prefs=new JButton("Preferences");
		have_prefs.addActionListener(this);
		have_prefs.setActionCommand("show_prefs");
		have_prefs.setBorder(new EmptyBorder(0,0,0,0));
		have_prefs.setVisible(false);

		JPanel east_pane=new JPanel(new FlowLayout(FlowLayout.RIGHT,0,0));
		east_pane.add(have_prefs);
		east_pane.add(state);

        add("Center",nameToClick);
        add("East",east_pane);
        setBorder(new EtchedBorder());

    }

    public boolean isActivated()
    {
        return(state.isSelected());
    }

    public void setActivated(boolean temp)
    {
        state.setSelected(temp);
        prefs.setPluginActivated(name,state.isSelected());
    }

    public int getId()
    {
        return(id);
    }

    public String getName()
    {
        return(name);
    }

    public String getCopyright()
    {
        return(copyright);
    }

    public String getDescription()
    {
        return(description);
    }

    public String getSummary()
    {
        return(summary);
    }

    public String getType()
    {
        return(type);
    }

    public String getFamily()
    {
        return(family);
    }

    public void actionPerformed(ActionEvent evt)
    {
        String command=evt.getActionCommand();

        if(command.equals("state"))
            prefs.setPluginActivated(name,state.isSelected());
           
        if(command.equals(name))
            gui.disp_plugin_property(this);

		if(command.equals("show_prefs"))
			plugin_prefs.show_prefs_box(parent);			
    }

	public PluginPrefs get_plugin_prefs()
	{
		return(plugin_prefs);
	}

	public void add_plugin_pref(String pref)
	{
		if(plugin_prefs==null)
			plugin_prefs=new PluginPrefs(name);
			
		plugin_prefs.add_pref(pref);
		have_prefs.setVisible(true);
	}
}

