/* NessusJ
 * Copyright (C) 1999 Alexis de Bernis <alexisb@cvs.nessus.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This program is a nessusd java client. For any information about
 * the nessus project visit <http://www.nessus.org>
 *
 *      PLEASE READ CAREFULLY NESSUS DOCUMENTATION BEFORE
 *                     USING THIS PROGRAM
 *
 */

/* importation directives has changed in newer JDK, uncomment
 * the ones you use if you have compilation troubles :
 * JDK up to 1.1.7 (Swing 1.0.x) : classes in com.sun.java.swing
 * JDK from 1.2 (Swing 1.1.x) : classes in javax.swing
 * You must change this in every .java file...
 */


import java.awt.*;
import java.awt.event.*;
import java.util.*;

import com.sun.java.swing.*;
import com.sun.java.swing.border.*;
//import javax.swing.*;
//import javax.swing.border.*;

public class PluginPrefs extends JPanel implements ActionListener
{
	private String name;
	private JDialog prefs_box;
	private Hashtable gadgets; // contains widget and Command to return

	PluginPrefs(String name)
	{
		super(new GridLayout(0,2));
		
		this.name=name;
		
		add(new Label("")); // put a white space at the beginning
		add(new Label("")); // of the prefs box panel
		
		gadgets=new Hashtable();
	}

	public void add_pref(String type)
	{
//		System.out.println("PluginPrefs :: add_pref : param = "+type);

		String gadget,label,default_value;
		gadget=type.substring(1,type.indexOf("]"));
		label=type.substring(type.indexOf("]")+2,type.indexOf(" <|> "));
		default_value=type.substring(type.indexOf(" <|> ")+5);

//		System.out.println("PluginPrefs :: add_pref : gadget        = |"+gadget+"|");
//		System.out.println("PluginPrefs :: add_pref : label         = |"+label+"|");
//		System.out.println("PluginPrefs :: add_pref : default_value = |"+default_value+"|");

		add_gadget(gadget,label,default_value);
	}


	private void add_gadget(String gadget, String label, String default_value)
	{
		add(new JLabel(label,JLabel.RIGHT));
		
		if(gadget.equals("entry"))
		{
			JTextField tmp=new JTextField(20);
			tmp.setText(default_value);
			add(tmp);
			gadgets.put(tmp,name+"[entry]:"+label+" <|> ");
			return;
		}

		if(gadget.equals("radio"))
		{
			int pos;
			ButtonGroup group=new ButtonGroup();
			
			String[] values=parse_entry_msg(default_value);
			
			JRadioButton tmp=new JRadioButton(values[0],true);
			group.add(tmp);
			add(tmp);
			add(new JLabel(""));

			for(int i=1;i<(values.length-1);i++)
			{
				tmp=new JRadioButton(values[i],false);
				group.add(tmp);
				add(tmp);
				add(new JLabel(""));
			}

			tmp=new JRadioButton(values[values.length-1],false);
			group.add(tmp);
			add(tmp);

			gadgets.put(group,name+"[radio]:"+label+" <|> ");
			return;
		}

		if(gadget.equals("checkbox"))
		{
			JCheckBox tmp=new JCheckBox("",default_value.equals("yes"));
			add(tmp);
			gadgets.put(tmp,name+"[checkbox]:"+label+" <|> ");
			return;
		}

		System.out.println("PluginPrefs :: add_gadget : unknown gadget "+gadget);
	}

	public void show_prefs_box(NessusJ parent)
	{
//		System.out.println("PluginPrefs :: show_prefs_box : started...");

		prefs_box=new JDialog(parent.get_gui(),"Preferences for "+name,true);
		JPanel prefs_title_pane,buttons_pane;
		JButton close_box;
		
		add(new Label(""));  // Put a white space at the end of
		add(new Label(""));  // the prefs box panel

		close_box=new JButton("Close");
		close_box.addActionListener(this);
		
		prefs_title_pane=new JPanel(new FlowLayout());
		prefs_title_pane.setBorder(new EtchedBorder());
		prefs_title_pane.add(new JLabel(name));
		
		buttons_pane=new JPanel(new FlowLayout());
		buttons_pane.add(close_box);
		buttons_pane.setBorder(new EtchedBorder());
		
		prefs_box.getContentPane().setLayout(new BorderLayout());
		prefs_box.getContentPane().add("North",prefs_title_pane);
		prefs_box.getContentPane().add("Center",this);
		prefs_box.getContentPane().add("South",buttons_pane);		
		
		prefs_box.pack();
		prefs_box.setVisible(true);
	}

	public void actionPerformed(ActionEvent evt)
	{
		prefs_box.dispose();
	}

	public String get_user_prefs()
	{
		String prefs="";
		Enumeration widgets=gadgets.keys();

		while(widgets.hasMoreElements())
		{
			Object tmp=widgets.nextElement();
			String widget_pref=(String) gadgets.get(tmp);
			
			if(tmp instanceof JTextField)
			{
				prefs=prefs+widget_pref+(((JTextField) tmp).getText())+"\n";
				continue;
			}
			
			if(tmp instanceof JCheckBox)
			{
				prefs=prefs+widget_pref+((((JCheckBox) tmp).isSelected())?"yes":"no")+"\n";
				continue;
			}
			
			if(tmp instanceof ButtonGroup)
			{
				JRadioButton tmp2=null;
				Enumeration buttons=((ButtonGroup) tmp).getElements();
				
				while(buttons.hasMoreElements())  // find which button is selected
				{
					tmp2=(JRadioButton) buttons.nextElement();
					if(tmp2.isSelected())
						break;
				}

				prefs=prefs+widget_pref+tmp2.getText()+"\n";
				continue;
			}

			System.out.println("PluginPrefs :: get_user_prefs : Unknown gadget "+tmp.toString());
		}

//		System.out.println("PluginPrefs :: get_user_prefs : prefs are \n"+prefs);
		return(prefs);
	}


    private String[] parse_entry_msg(String msg)
    {
        int pos=-1,num=0;

//		System.out.println("NessusJ :: parse_ntp_msg : parsing "+msg);

        while((pos=msg.indexOf(";",pos+1))!=-1)
		    num++;
			
        String[] res=new String[num+1];

        res[0]=msg.substring(0,(pos=msg.indexOf(";")));
        res[num]=msg.substring(msg.lastIndexOf(";")+1);

        for(int i=1;i<num;i++)
		{
//			System.out.println("NessusJ :: parse_ntp_msg : pos="+pos+" num="+num);
            res[i]=msg.substring(pos+1,(pos=msg.indexOf(";",pos+1)));
		}
		
        return(res);
    }
}
		
