/* NessusJ
 * Copyright (C) 1999 Alexis de Bernis <alexisb@cvs.nessus.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This program is a nessusd java client. For any information about
 * the nessus project visit <http://www.nessus.org>
 *
 *      PLEASE READ CAREFULLY NESSUS DOCUMENTATION BEFORE
 *                     USING THIS PROGRAM
 *
 */

/* importation directives has changed in newer JDK, uncomment
 * the ones you use if you have compilation troubles :
 * JDK up to 1.1.7 (Swing 1.0.x) : classes in com.sun.java.swing
 * JDK from 1.2 (Swing 1.1.x) : classes in javax.swing
 * You must change this in every .java file...
 */


import java.awt.*;
import java.util.Vector;
import java.io.*;
import java.awt.event.*;

import com.sun.java.swing.*;
import com.sun.java.swing.border.*;
//import javax.swing.*;
//import javax.swing.border.*;


class Report extends JFrame implements WindowListener,ActionListener
{
    JButton close,save,saveHTML;
    Box host,port;
    JScrollPane hostsPane,portsPane;
    JTextArea description;
    JPanel button,lists;
    Vector hosts;
    String curHost=" ";
    String curPort="0";
	PicsLib pics;
	NessusJ parent;


    Report(NessusJ parent, Vector hosts)
    {
        super("Report");

        this.hosts=hosts;
        this.parent=parent;
		pics=parent.get_pics_lib();

        addWindowListener(this);
        close=new JButton("Close");
        close.addActionListener(this);
        save=new JButton("Save");
        save.addActionListener(this);
        saveHTML=new JButton("Save to HTML");
        saveHTML.addActionListener(this);

        host=new Box(BoxLayout.Y_AXIS);
        port=new Box(BoxLayout.Y_AXIS);
        hostsPane=new JScrollPane(host);
        hostsPane.setBorder(new TitledBorder(new EtchedBorder(),"Hosts"));
        portsPane=new JScrollPane(port);
        portsPane.setBorder(new TitledBorder(new EtchedBorder(),"Ports"));

        description=new JTextArea("",10,40);
        description.setEditable(false);
        description.setLineWrap(true);
        description.setWrapStyleWord(true);
		JScrollPane description_scrollpane=new JScrollPane(description);

        button=new JPanel(new FlowLayout());
        button.add(close);
        button.add(save);
        button.add(saveHTML);

        lists=new JPanel(new GridLayout(0,2));
        lists.add(hostsPane);
        lists.add(portsPane);

        getContentPane().setLayout(new BorderLayout());
        getContentPane().add("North",lists);
        getContentPane().add("Center",description_scrollpane);
        getContentPane().add("South",button);

        refreshHosts();

        pack();
        setResizable(true);
		setLocation(500,100);
        setVisible(true);
    }

    public void refreshHosts()
    {
        host.removeAll();

        for(int i=0;i<hosts.size();i++)
        {
            String icon="flag_green";
            Host current=(Host) hosts.elementAt(i);

            if(current.getInfos().size()!=0)
                icon="flag_orange";

            if(current.getHoles().size()!=0)
                icon="flag_red";

            JButton toAdd=new JButton(current.getName(),pics.get_pic(icon));
            toAdd.setBorder(new EmptyBorder(0,0,0,0));
            toAdd.addActionListener(this);

            host.add(toAdd);
        }

        validateTree();
		repaint();
    }



    public void refreshDescription(String port2show)
    {
        curPort=port2show;
        Host current=getHost(curHost);
        Vector holes=current.getHoles();
        Vector infos=current.getInfos();

        clearText(description);

        description.append("Holes :\n");

        for(int i=0;i<holes.size();i++)
        {
            if(((Hole) holes.elementAt(i)).getPort().equals(port2show))
                description.append(((Hole) holes.elementAt(i)).getDescription()+"\n");
        }

        description.append("Infos :\n");     //Separate infos and holes

        for(int i=0;i<infos.size();i++)
        {
            if(((Info) infos.elementAt(i)).getPort().equals(port2show))
                description.append(((Info) infos.elementAt(i)).getDescription()+"\n");
        }
    }
        



    public void refreshPort()
    {
        String toAddIcon,toAddText;

        port.removeAll();

        Host current=getHost(curHost);

        Vector ports=current.getOpenPorts();

        for(int i=0;i<ports.size();i++)
        {
            String actual=(String) ports.elementAt(i);

            toAddIcon="flag_green";
            toAddText=String.valueOf(actual);

            if(isInfo(current,actual))
                toAddIcon="flag_orange";

            if(isHole(current,actual))
                toAddIcon="flag_red";

            JButton toAddButton=new JButton(toAddText,pics.get_pic(toAddIcon));
            toAddButton.setBorder(new EmptyBorder(0,0,0,0));
            toAddButton.addActionListener(this);
            port.add(toAddButton);
        }

//        validateTree();
        pack();
		repaint();
    }

    private boolean isHole(Host host,String toTest)
    {
        
        Vector holes=host.getHoles();

        for(int i=0;i<holes.size();i++)
        {
            if(((Hole) holes.elementAt(i)).getPort().equals(toTest))
                return(true);
        }
        return(false);
    }

    private boolean isInfo(Host host,String toTest)
    {
        
        Vector infos=host.getInfos();

        for(int i=0;i<infos.size();i++)
        {
            if(((Info) infos.elementAt(i)).getPort().equals(toTest))
                return(true);
        }
        return(false);
    }

    private Host getHost(String name)
    {
        for(int i=0;i<hosts.size();i++)
        {
            if(((Host) hosts.elementAt(i)).getName().equals(name))
                return((Host) hosts.elementAt(i));
        }

        return(null);
    }

    private void clearText(JTextArea area)
    {
        area.setText("");
    }

    private void saveReport()
    {
        FileDialog dialog=new FileDialog(this,"Save Report as...",FileDialog.SAVE);
        String filename;
        DataOutputStream toFile;

        dialog.setFile("*.nsr");
        dialog.show();

        if((filename=dialog.getFile())==null)
            return;

        try
        {
            toFile=new DataOutputStream(new FileOutputStream(dialog.getDirectory()+filename));
        }catch(IOException e)
        {
            System.out.println("Error: impossible to create file : "+filename);
            return;
        }

        for(int i=0;i<hosts.size();i++)
        {
            String name=((Host) hosts.elementAt(i)).getName();
            Vector infos=((Host) hosts.elementAt(i)).getInfos();
            Vector holes=((Host) hosts.elementAt(i)).getHoles();

            for(int j=0;j<infos.size();j++)
            {
                String port=((Info) infos.elementAt(j)).getPort();

                String toWrite=name+" "+
                               port+" "+
                               "INFO"+" "+
                               ((Info) infos.elementAt(j)).getDescription();
                toWrite=toWrite.replace('\n',';')+"\n";

                port=null;

                try
                {
                    toFile.writeBytes(toWrite);
                }catch(IOException e)
                {
                    System.out.println("Error : impossible to write report");
                    return;
                }

                toWrite=null;
            }

            for(int j=0;j<holes.size();j++)
            {
                String port=((Hole) holes.elementAt(j)).getPort();

                String toWrite=name+" "+
                               port+" "+
                               "REPORT"+" "+ 
                               ((Hole) holes.elementAt(j)).getDescription();
                toWrite=toWrite.replace('\n',';')+"\n";
                try
                {
                    toFile.writeBytes(toWrite);
                }catch(IOException e)
                {
                    System.out.println("Error : impossible to write report");
                    return;
                }

                toWrite=null;
            }

            name=null;
            infos=null;
            holes=null;
        }
    }

    public void back()
    {
        dispose();
    }

    private void saveReportToHTML()
    {

        FileDialog dialog=new FileDialog(this,"Save Report as...",FileDialog.SAVE);
        String filename;
        DataOutputStream toFile;

        dialog.setFile("*.html");
        dialog.show();

        if((filename=dialog.getFile())==null)
            return;

        try
        {
            toFile=new DataOutputStream(new FileOutputStream(dialog.getDirectory()+filename));
        }catch(IOException e)
        {
            System.out.println("Error: impossible to create file : "+filename);
            return;
        }

        try
        {
            toFile.writeBytes("<HTML>\n<HEAD>\n<TITLE>Nessus Report</TITLE>\n</HEAD>\n");
            toFile.writeBytes("<BODY BGCOLOR=\"#FFFFFF\" TEXT=\"#000000\">\n");
            toFile.writeBytes("<CENTER><U><H1>Nessus Report</H1></U></CENTER>\n");
            toFile.writeBytes("<BR>\n<BR>\n");

            for(int i=0;i<hosts.size();i++)
            {

                toFile.writeBytes("<H2><B>Host : "+((Host) hosts.elementAt(i)).getName()+"</B></H2>\n");
                toFile.writeBytes("<BR>\n<UL>\n");
                toFile.writeBytes("<B><LI>Holes : </B>\n");
                toFile.writeBytes("<UL>\n");
            
                Vector holes=((Host) hosts.elementAt(i)).getHoles();

                for(int j=0;j<holes.size();j++)
                {
                    String portNumber=((Hole) holes.elementAt(j)).getPort();

                    toFile.writeBytes("<LI>"+portNumber+" : "+((Hole) holes.elementAt(j)).getDescription()+"\n");
                }

                toFile.writeBytes("</UL>\n<B><LI>Infos :</B>\n<UL>\n");

                Vector infos=((Host) hosts.elementAt(i)).getInfos();

                for(int j=0;j<infos.size();j++)
                {
                    String portNumber=((Info) infos.elementAt(j)).getPort();

                    toFile.writeBytes("<LI>"+portNumber+" : "+((Info) infos.elementAt(j)).getDescription()+"\n");
                }

                toFile.writeBytes("</UL>\n</UL>\n<HR>\n");

            }

                toFile.writeBytes("<I>This file was automagically created by <A HREF=\"http://www.nessus.org/nessusj\">NessusJ</A>.</I>\n");
                toFile.writeBytes("</BODY>\n</HTML>");
        } catch(IOException e)
        {
            System.out.println("Error writing report to HTML : "+e);
            return;
        }
    }

    public void windowClosing(WindowEvent evt)
    {
        back();
    }

    public void windowClosed(WindowEvent evt ) { }
    public void windowOpened(WindowEvent evt) { }
    public void windowOpening(WindowEvent evt ) { }
    public void windowIconified(WindowEvent evt ) { }
    public void windowActivated(WindowEvent evt ) { }
    public void windowDeiconified(WindowEvent evt ) { }
    public void windowDeactivated(WindowEvent evt ) { }

    public void actionPerformed(ActionEvent evt)
    {
        String command=evt.getActionCommand();

        if(command.equals("Close"))
        {
            back();
            return;
        }

        if(command.equals("Save"))
        {
            saveReport();
            return;
        }

        if(command.equals("Save to HTML"))
        {
            saveReportToHTML();
            return;
        }
        
        Host temp;

        if((temp=getHost(command))!=null)
        {
            curHost=command;
            refreshPort();
            return;

        } else
        {
            refreshDescription(command);
        }
    }

    public String getCurrentHost()
    {
        return(curHost);
    }

    public String getCurrentPort()
    {
        return(curPort);
    }
}
